"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const Deserializer_1 = require("./Deserializer");
const _1 = require("../");
const fixtureSimple = '{"firstName":"Dan","lastName":"Revah","age":28,"isHere":true,"birthDate":"2018-07-15T05:35:03.000Z"}';
const fixtureChild = `{"child":${fixtureSimple}}`;
const fixtureChildren = `{"children":[${fixtureSimple}, ${fixtureSimple}]}`;
const fixtureDeserialize = `{"prop":"name"}`;
class Simple {
}
__decorate([
    _1.Type(Date),
    __metadata("design:type", Date)
], Simple.prototype, "birthDate", void 0);
exports.Simple = Simple;
class SimpleChild {
}
__decorate([
    _1.Type(Simple),
    __metadata("design:type", Simple)
], SimpleChild.prototype, "child", void 0);
exports.SimpleChild = SimpleChild;
class SimpleChildArr {
}
__decorate([
    _1.Type([Simple]),
    __metadata("design:type", Array)
], SimpleChildArr.prototype, "children", void 0);
exports.SimpleChildArr = SimpleChildArr;
class CauseException {
}
__decorate([
    _1.Type([Simple, Simple]),
    __metadata("design:type", Array)
], CauseException.prototype, "children", void 0);
exports.CauseException = CauseException;
class Prop {
    constructor(name) {
        this.name = name;
    }
}
exports.Prop = Prop;
class SimpleDeserializer {
}
__decorate([
    _1.Deserializer((value) => new Prop('my-prop-' + value)),
    __metadata("design:type", Prop)
], SimpleDeserializer.prototype, "prop", void 0);
exports.SimpleDeserializer = SimpleDeserializer;
describe('Deserializer', () => {
    it('should equal basic type with primitives and Date', () => {
        const simple = Deserializer_1.deserialize(fixtureSimple, Simple);
        chai_1.expect(simple.firstName).to.equal('Dan');
        chai_1.expect(simple.lastName).to.equal('Revah');
        chai_1.expect(simple.age).to.equal(28);
        chai_1.expect(simple.isHere).to.equal(true);
        chai_1.expect(simple.birthDate).instanceof(Date);
    });
    it('should equal with child hierarchy', () => {
        const simple = Deserializer_1.deserialize(fixtureChild, SimpleChild);
        chai_1.expect(simple.child).to.be.instanceof(Simple);
        chai_1.expect(simple.child.firstName).to.equal('Dan');
        chai_1.expect(simple.child.lastName).to.equal('Revah');
        chai_1.expect(simple.child.age).to.equal(28);
        chai_1.expect(simple.child.isHere).to.equal(true);
        chai_1.expect(simple.child.birthDate).instanceof(Date);
    });
    it('should equal with array as child hierarchy', () => {
        const simple = Deserializer_1.deserialize(fixtureChildren, SimpleChildArr);
        chai_1.expect(simple.children[0]).to.be.instanceof(Simple);
        chai_1.expect(simple.children[0].firstName).to.equal('Dan');
        chai_1.expect(simple.children[0].lastName).to.equal('Revah');
        chai_1.expect(simple.children[0].age).to.equal(28);
        chai_1.expect(simple.children[0].isHere).to.equal(true);
        chai_1.expect(simple.children[0].birthDate).instanceof(Date);
        chai_1.expect(simple.children[1]).to.be.instanceof(Simple);
        chai_1.expect(simple.children[1].firstName).to.equal('Dan');
        chai_1.expect(simple.children[1].lastName).to.equal('Revah');
        chai_1.expect(simple.children[1].age).to.equal(28);
        chai_1.expect(simple.children[1].isHere).to.equal(true);
        chai_1.expect(simple.children[1].birthDate).instanceof(Date);
    });
    it('should throw an error if have more than once value in @Type array annotation', () => {
        chai_1.expect(() => Deserializer_1.deserialize(fixtureChildren, CauseException)).to.throw('`@Type` can only be defined with a single value, or an array with a single value. for ex: `@Type(User)` or `@Type([User])`');
    });
    it('should use the `@Deserialize` function on a property while deserialize', () => {
        const simpleDes = Deserializer_1.deserialize(fixtureDeserialize, SimpleDeserializer);
        chai_1.expect(simpleDes.prop.name).to.equal('my-prop-name');
    });
});
//# sourceMappingURL=Deserializer.spec.js.map